////////////////////////////////////////////////////////////////////////////////
// aulslib.h
// AviUtlvOCgҏWXNvgpDLLJ⏕
////////////////////////////////////////////////////////////////////////////////

#pragma once

#ifndef AULSLIB_H
#define AULSLIB_H

#include <windows.h>

namespace auls {\



template <typename T, size_t N>
size_t ElemOf(T (&v)[N])
{
	return N;
}

template <typename T>
T& Max(T& v1, T& v2)
{
	return v1 >= v2 ? v1 : v2;
}

template <typename T>
T& Min(T& v1, T& v2)
{
	return v1 <= v2 ? v1 : v2;
}

template <typename T>
void Swap(T& v1, T& v2)
{
	T temp = v1;
	v1 = v2;
	v2 = temp;
}

inline void StringClear(LPTSTR buf, size_t size) { if(size > 0) *buf = TEXT('\0'); }

inline BOOL OpenFileDialog(LPTSTR buf, size_t size, HWND owner, LPCTSTR filter, LPCTSTR directory, LPCTSTR caption)
{
	OPENFILENAME ofn = {};
	ofn.lStructSize     = sizeof(ofn);
	ofn.hwndOwner       = owner;
	ofn.lpstrFilter     = filter;
	ofn.lpstrFile       = buf;
	ofn.nMaxFile        = size;
	ofn.lpstrInitialDir = directory;
	ofn.lpstrTitle      = caption;
	ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	return GetOpenFileName(&ofn);
}

inline BOOL SaveFileDialog(LPTSTR buf, size_t size, HWND owner, LPCTSTR filter, LPCTSTR directory, LPCTSTR caption)
{
	OPENFILENAME ofn = {};
	ofn.lStructSize     = sizeof(ofn);
	ofn.hwndOwner       = owner;
	ofn.lpstrFilter     = filter;
	ofn.lpstrFile       = buf;
	ofn.nMaxFile        = size;
	ofn.lpstrInitialDir = directory;
	ofn.lpstrTitle      = caption;
	ofn.Flags           = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt     = ""; // NULL ƊgqtȂ
	return GetSaveFileName(&ofn);
}

// Œ蒷pXNX
class CFixedPath {
public:
	TCHAR buf[MAX_PATH];

	CFixedPath(void) { StringClear(buf, ElemOf(buf)); }

	operator LPCTSTR() { return buf; }

	void Clear(void) { StringClear(buf, ElemOf(buf)); }

	BOOL OpenFileDialog(HWND owner, LPCTSTR filter, LPCTSTR directory, LPCTSTR caption)
	{
		return auls::OpenFileDialog(buf, ElemOf(buf), owner, filter, directory, caption);
	}

	BOOL SaveFileDialog(HWND owner, LPCTSTR filter, LPCTSTR directory, LPCTSTR caption)
	{
		return auls::SaveFileDialog(buf, ElemOf(buf), owner, filter, directory, caption);
	}
};

// t@CNX
class CFile {
public:
	HANDLE file;

	CFile(void) { file = INVALID_HANDLE_VALUE; }
	~CFile(void) { if(IsValid()) CloseHandle(file); }

	operator bool() { return IsValid(); }
	operator HANDLE() { return file; }

	bool IsValid(void) { return file != INVALID_HANDLE_VALUE; }

	bool OpenExisting(LPCTSTR filename)
	{
		if(!Close()) return false;
		file = CreateFile(filename, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		return IsValid();
	}

	bool CreateAlways(LPCTSTR filename)
	{
		if(!Close()) return false;
		file = CreateFile(filename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		return IsValid();
	}

	bool Close(void)
	{
		if(!IsValid()) return true;
		if(!CloseHandle(file)) return false;
		file = INVALID_HANDLE_VALUE;
		return true;
	}

	DWORD Size(void) { return GetFileSize(file, NULL); }
	DWORD PtrPos(void) { return SetFilePointer(file, 0, NULL, FILE_CURRENT); }

	BOOL Read(LPVOID buf, DWORD size)
	{
		return ReadFile(file, buf, size, &size, NULL);
	}

	BOOL Write(LPCVOID buf, DWORD size)
	{
		return WriteFile(file, buf, size, &size, NULL);
	}
};

// EBhENX
class CWindow {
public:
	HWND hwnd;

	CWindow(void) { hwnd = NULL; }
	~CWindow(void) {}

	operator bool() { return hwnd != NULL; }
	operator HWND() { return hwnd; }

	bool Create(LPCTSTR classname, LPCTSTR title, DWORD style, DWORD exstyle, int x, int y, int width, int height, HWND parent, HMENU menu, HINSTANCE inst, LPVOID param)
	{
		hwnd = CreateWindowEx(exstyle, classname, title, style, x, y, width, height, parent, menu, inst, param);
		return hwnd != NULL;
	}

	bool Destroy(void)
	{
		if(!DestroyWindow(hwnd)) return false;
		hwnd = NULL;
		return true;
	}

	BOOL Enable(BOOL flag) { return EnableWindow(hwnd, flag); }
	BOOL Redraw(BOOL erase) { return InvalidateRect(hwnd, NULL, erase); }
};

// yNX
class CPen {
public:
	HPEN pen;

	CPen(void) { pen = NULL; }
	~CPen(void) {}
	bool IsValid(void) { return pen != NULL; }

	operator bool() { return IsValid(); }
	operator HPEN() { return pen; }

	bool Create(int style, int width, COLORREF color)
	{
		if(!Delete()) return false;
		pen = CreatePen(style, width, color);
		return pen != NULL;
	}

	bool Create(COLORREF color) { return Create(PS_SOLID, 1, color); }
	bool Delete(void)
	{
		if(!IsValid()) return true;
		if(!DeleteObject(pen)) return false;
		pen = NULL;
		return true;
	}
};

class PixelDataArgs {
public:
	DWORD *data;
	DWORD w, h;

	PixelDataArgs(lua_State *L, int index)
	{
		data = (DWORD*)lua_touserdata(L, index);
		if(!data) luaL_typerror(L, index, "userdata");
		w = (DWORD)lua_tonumber(L, index+1);
		h = (DWORD)lua_tonumber(L, index+2);
	}
};

class RgbaValue {
public:
	union {
		DWORD val;
		struct { BYTE b, g, r, a; };
	};

	RgbaValue(DWORD v) { val = v; }

	operator DWORD() { return val; }

	RgbaValue& operator= (DWORD v) { val = v; return *this; }
};



} // namespace auls

#endif // #ifndef AULSLIB_H